import { Packet } from 'src/net/packet';
import { PlayerSocket } from 'src/types/data-socket';
import { Humanoid } from './humanoid.object';
import { Group } from './group.object';
import { Vector3 } from 'three';

export class Player {
  public character: Group;
  public controller: Humanoid;
  public initialized = false;

  constructor(
    public socket: PlayerSocket,
    public id: string,
    public name: string,
    private token: string,
  ) {}

  public send(packet: Packet): void;
  public send(packet: Buffer): void;
  public send(packet: unknown): void {
    if (packet instanceof Packet) {
      return this.socket.send(packet.toBuffer());
    }
    this.socket.send(packet as Buffer);
  }

  public createPlayerCharacter(pos = new Vector3(0, 1, 0)) {
    const group = new Group();
    group.name = this.name;
    group.position.copy(pos);
    this.character = group;

    const humanoid = new Humanoid();
    humanoid.uuid = this.id;
    group.add(humanoid);
    this.controller = humanoid;

    return group;
  }
}
