import { Matrix4, Vector3 } from 'three';
import { Brick } from './brick.object';
import type Rapier from '@dimforge/rapier3d-compat';
import { WedgeCornerGeometry } from './geometries';

export class WedgeCorner extends Brick {
  public objectType = 'WedgeCorner';
  public name = this.objectType;

  public static geometry = new WedgeCornerGeometry();

  protected override createCollider(
    factory: typeof Rapier,
    world: Rapier.World,
    body?: Rapier.RigidBody,
  ) {
    const scale = this.getWorldScale(new Vector3());
    const mat = new Matrix4();
    mat.makeScale(...scale.toArray());
    const points = WedgeCorner.geometry
      .getAttribute('position')
      .clone()
      .applyMatrix4(mat)?.array as Float32Array;
    const collider = factory.ColliderDesc.convexMesh(points)!;
    return world.createCollider(collider, body);
  }
}
