import { NestFactory } from '@nestjs/core';
import { AppModule } from './app.module';
import { WsAdapter } from './net/ws-adapter';
import { ConfigService } from '@nestjs/config';
import { Logger } from '@nestjs/common';

async function bootstrap() {
  const app = await NestFactory.create(AppModule);
  const config = app.get<ConfigService>(ConfigService);
  const appPort = config.get<number>('server.statusPort');
  const gatewayPort = config.get<number>('server.gatewayPort');
  const corsOrigin = config.get('server.corsOrigin');

  app.enableCors({
    origin: corsOrigin,
    credentials: true,
  });

  app.useWebSocketAdapter(new WsAdapter(app, gatewayPort, corsOrigin));
  await app.listen(appPort);

  Logger.log(`Server status API: ${await app.getUrl()}`, 'bootstrap');
}
bootstrap();
